package no.ntnu.item.sso;

import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import no.ntnu.item.sso.enums.Country;
import no.ntnu.item.sso.enums.Gender;
import no.ntnu.item.sso.enums.Language;

import javax.validation.constraints.*;

/**
 * As a minimum, a {@link User} entity contains the fields of OpenID's
 * <a href="http://openid.net/specs/openid-simple-registration-extension-1_0.html#response_format">SREG Response Format</a>
 * 
 * @author jogrimst
 */
@Entity
//@no.ntnu.item.sso.validation.annotations.User
public class UserProfile extends BaseEntity {

    private static final long serialVersionUID = 1L;

    private String nickname;

    @NotNull
    @Column(nullable=false/*, unique=true*/)
    private String email;

    private String fullName;

    @Past
    @Temporal(TemporalType.DATE)
    private Calendar dateOfBirth;

    private Gender gender;

//	@PostCode
    private String postcode;

    private Country country;

    private Language preferredLanguage;

//	private TimeZone timeZone; // For example, "Europe/Paris" or "America/Los_Angeles"

    @Past
    @Temporal(TemporalType.DATE)
    private Calendar lastSignOn;

//    private List<OpenIdIdentifier> openIdIdentifiers = new ArrayList<OpenIdIdentifier>();

    /**
     * A Set will prevent duplicate elements from being inserted.
     */
    @ElementCollection
    private Set<String> openIdIdentifiers = new HashSet<String>();

    public String getNickname() {
            return nickname;
    }

    public void setNickname(String nickname) {
            this.nickname = nickname;
    }

    public String getEmail() {
            return email;
    }

    public void setEmail(String emailAddress) {
            this.email = emailAddress;
    }

    public String getFullName() {
            return fullName;
    }

    public void setFullName(String fullName) {
            this.fullName = fullName;
    }

    public Calendar getDateOfBirth() {
            return dateOfBirth;
    }

    public void setDateOfBirth(Calendar dateOfBirth) {
            this.dateOfBirth = dateOfBirth;
    }

    public Gender getGender() {
            return gender;
    }

    public void setGender(Gender gender) {
            this.gender = gender;
    }

    public String getPostcode() {
            return postcode;
    }

    public void setPostcode(String postcode) {
            this.postcode = postcode;
    }

    public Country getCountry() {
            return country;
    }

    public void setCountry(Country country) {
            this.country = country;
    }

    public void setPreferredLanguage(Language preferredLanguage) {
            this.preferredLanguage = preferredLanguage;
    }

    public Language getPreferredLanguage() {
            return preferredLanguage;
    }

//	public TimeZone getTimeZone() {
//		return timeZone;
//	}
//	
//	public void setTimeZone(TimeZone timeZone) {
//		this.timeZone = timeZone;
//	}

    public Calendar getLastSignOn() {
        return lastSignOn;
    }

    public void setLastSignOn(Calendar lastSignOn) {
        this.lastSignOn = lastSignOn;
    }

    public Set<String> getOpenIdIdentifiers() {
        return openIdIdentifiers;
    }

    public void setOpenIdIdentifiers(Set<String> openIdIdentifiers) {
        this.openIdIdentifiers = openIdIdentifiers;
    }
}
