package no.ntnu.item.sso.web.user;

import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.List;
import javax.servlet.http.HttpServletRequest;

import no.ntnu.item.sso.UserProfile;

import no.ntnu.item.sso.dao.UserProfileDao;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.openid.OpenIDAttribute;
import org.springframework.security.openid.OpenIDAuthenticationToken;
import org.springframework.security.web.authentication.WebAuthenticationDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

/**
 * @author jogrimst
 */
@Controller
@RequestMapping("/user")
public class AccountController {

    private final Logger logger = Logger.getLogger(getClass());

    private UserProfile activeUserProfile;
    private boolean isFirstViewAfterSignOn = true;
	
    @Autowired
    private UserProfileDao userProfileDao;

    @RequestMapping(value = "/account", method = RequestMethod.GET)
    public void accountHandler(ModelMap model) {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        logger.info("authentication: " + authentication);

        User principal = (User) authentication.getPrincipal();
        logger.info("principal.getUsername(): " + principal.getUsername());

        WebAuthenticationDetails webAuthenticationDetails = (WebAuthenticationDetails) authentication.getDetails();
        logger.info("webAuthenticationDetails: " + webAuthenticationDetails);

        boolean isNewUserProfile = false;
        // Check if a user profile with the given OpenID Identifier already exists in the database
        UserProfile userProfile = userProfileDao.readByOpenIdIdentifier(principal.getUsername());

        if (userProfile == null) {
            // Store new user profile

            userProfile = new UserProfile();
            userProfile.getOpenIdIdentifiers().add(principal.getUsername());

            OpenIDAuthenticationToken openIDAuthenticationToken = (OpenIDAuthenticationToken) authentication;
            List<OpenIDAttribute> openIDAttributes = openIDAuthenticationToken.getAttributes();
            for (OpenIDAttribute openIDAttribute : openIDAttributes) {
                if ("email".equals(openIDAttribute.getName())) {
                    userProfile.setEmail(openIDAttribute.getValues().get(0));
                }
            }
            if (userProfile.getEmail() == null) {
                // Fallback address (for testing purposes)
                userProfile.setEmail("jogrimst@stud.ntnu.no");
            }

            UserProfile userProfileWithMatchingEmail = userProfileDao.readByEmail(userProfile.getEmail());
            if (userProfileWithMatchingEmail == null) {
                isNewUserProfile = true;
                userProfileDao.create(userProfile);
            } else {
                userProfileWithMatchingEmail.getOpenIdIdentifiers().add(principal.getUsername());
                userProfile = userProfileWithMatchingEmail;
            }
        }

        activeUserProfile = userProfile;

        userProfile.setLastSignOn(Calendar.getInstance());

        model.addAttribute("userProfile", userProfile);
        model.addAttribute("isNewUserProfile", isNewUserProfile);
        if (isFirstViewAfterSignOn) {
            model.addAttribute("isFirstViewAfterSignOn", isFirstViewAfterSignOn);
        }
        isFirstViewAfterSignOn = false;
    }

    @RequestMapping(value = "/manage-openids", method = RequestMethod.GET)
    public void accountHandlerOpenId(ModelMap model) {
        model.addAttribute("userProfile", activeUserProfile);
        model.addAttribute("openIdOfCurrentUserProfile", ((User) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername());
    }

//    @RequestMapping(value = "/manage-openids/add-an-openid", method = RequestMethod.GET)
    @RequestMapping(value = "/manage-openids/add-an-openid", method = RequestMethod.POST)
    public String accountHandlerAddAnOpenId(ModelMap model, HttpServletRequest request) {
        String newOpenId = request.getParameter("openid_identifier"); // http://openid.net/specs/openid-authentication-2_0.html#initiation
        activeUserProfile.getOpenIdIdentifiers().add(newOpenId);
        userProfileDao.update(activeUserProfile);
        model.addAttribute("userProfile", activeUserProfile);
        model.addAttribute("openIdOfCurrentUserProfile", ((User) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername());
        model.addAttribute("isOpenIdWasAdded", true);
        return "user/manage-openids";
    }

    @RequestMapping(value = "/manage-openids/delete-an-openid", method = RequestMethod.GET)
    public String accountHandlerDeleteAnOpenId(ModelMap model, HttpServletRequest request) {
        String newOpenId = request.getParameter("openid_identifier"); // http://openid.net/specs/openid-authentication-2_0.html#initiation
        activeUserProfile.getOpenIdIdentifiers().remove(newOpenId);
        userProfileDao.update(activeUserProfile);
        model.addAttribute("userProfile", activeUserProfile);
        model.addAttribute("openIdOfCurrentUserProfile", ((User) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername());
        model.addAttribute("isOpenIdWasDeleted", true);
        return "user/manage-openids";
    }

//    public static void main(String[] args) throws NoSuchAlgorithmException, UnsupportedEncodingException {
//        MessageDigest messageDigest = MessageDigest.getInstance("MD5");
//        String md5Hash = new BigInteger(1, messageDigest.digest("jogrimst@gmail.com".getBytes())).toString(16);
//        System.out.println(md5Hash);
//    }
}
